//----------------------------------------------------------------------
// File : AsDemoApp.h
// Desc : Asura Demo Application
// Copyright(c) Project Asura All right reserved.
//----------------------------------------------------------------------

//----------------------------------------------------------------------
// History
//----------------------------------------------------------------------
// <---> 2010/12/04 [Pocol] VK쐬.
// <End>

#ifndef __ASURA_DEMO_APP_H__
#define __ASURA_DEMO_APP_H__

//----------------------------------------------------------------------
// Includes
//----------------------------------------------------------------------
#include <vector>
#include <cassert>

//-----------------------------------------------------------------------
// null Definition
//-----------------------------------------------------------------------
const class 
{
	//-------------------------------------------------------------------
	// list of friend classes
	//-------------------------------------------------------------------
	/* NOTHING */

public:
	//-------------------------------------------------------------------
	// public variables
	//-------------------------------------------------------------------
	/* NOTHING */

	//-------------------------------------------------------------------
	// public methods
	//-------------------------------------------------------------------
	template<class T>            operator T* () const { return 0; }
	template<class T0, class T1> operator T1 T0::*() const { return 0; }

protected:
	//-------------------------------------------------------------------
	// protected variables
	//-------------------------------------------------------------------
	/* NOTHING */

	//-------------------------------------------------------------------
	// protected methods
	//-------------------------------------------------------------------
	/* NOTHING */

private:
	//-------------------------------------------------------------------
	// private variables
	//-------------------------------------------------------------------
	/* NOTHING */

	//-------------------------------------------------------------------
	// private methods
	//-------------------------------------------------------------------
	void operator& () const {}
} null;


//-----------------------------------------------------------------------
// Macro
//-----------------------------------------------------------------------
#ifndef SAFE_DELETE
#define SAFE_DELETE(x) { if (x) { delete (x); (x)=null; } }
#endif

#ifndef SAFE_DELETE_ARRAY
#define SAFE_DELETE_ARRAY(x) { if (x) { delete[] (x); (x)=null; } }
#endif

#ifndef SAFE_RELEASE
#define SAFE_RELEASE(x) { if (x) { (x)->Release(); (x)=null; } }
#endif

#ifndef DLOG
#if defined(DEBUG) || defined(_DEBUG)
#define DLOG(x, ...) std::printf( "[File: %s, Line: %d] "x"\n", __FILE__, __LINE__, ##__VA_ARGS__ );
#else
#define DLOG(x, ...)
#endif
#endif

#ifndef ELOG
#define ELOG(x, ...) std::printf( "[File: %s, Line: %d] "x"\n", __FILE__, __LINE__, ##__VA_ARGS__ );
#endif

/////////////////////////////////////////////////////////////////////////
// Global Functions
/////////////////////////////////////////////////////////////////////////
bool IsGLError();


namespace Schaft {
namespace Asura {
namespace SampleProgram {

/////////////////////////////////////////////////////////////////////////
// IRenderer class
/////////////////////////////////////////////////////////////////////////
class IRenderer
{
	//-------------------------------------------------------------------
	// list of friend classes
	//-------------------------------------------------------------------
	/* NOTHING */

public:
	//-------------------------------------------------------------------
	// public variables
	//-------------------------------------------------------------------
	/* NOTHING */

	//-------------------------------------------------------------------
	// public methods
	//-------------------------------------------------------------------
	virtual bool Initialize() = 0;
	virtual void PreRender() = 0;
	virtual void Render() = 0;
	virtual void RenderAlpha() = 0;
	virtual void PostRender() = 0;
	virtual void Release() = 0;

protected:
	//-------------------------------------------------------------------
	// protected variables
	//-------------------------------------------------------------------
	/* NOTHING */

	//-------------------------------------------------------------------
	// protected methods
	//-------------------------------------------------------------------
	/* NOTHING */

private:
	//-------------------------------------------------------------------
	// private variables
	//-------------------------------------------------------------------
	/* NOTHING */

	//-------------------------------------------------------------------
	// private methods
	//-------------------------------------------------------------------
	/* NOTHING */
};

/////////////////////////////////////////////////////////////////////////
// DemoApp class
/////////////////////////////////////////////////////////////////////////
class DemoApp
{
	//-------------------------------------------------------------------
	// list of friend classes
	//-------------------------------------------------------------------
	/* NOTHING */

public:
	//-------------------------------------------------------------------
	// public variables
	//-------------------------------------------------------------------
	/* NOTHING */

	//-------------------------------------------------------------------
	// public methods
	//-------------------------------------------------------------------
	static DemoApp* GetInstance();
	virtual int		Register( IRenderer* pRenderer ) = 0;
	virtual void	Run() = 0;
	virtual int     GetWidth() = 0;
	virtual int     GetHeight() = 0;
	virtual float   GetAspectRatio() = 0;
	virtual bool    IsKeyDown( int keycode ) = 0;
	virtual bool    IsKeyUp( int keycode ) = 0;
	virtual bool    IsMouseDown( int button ) = 0;
	virtual bool    IsMouseUp( int buttton ) = 0;
	virtual int     GetCursorX() = 0;
	virtual int     GetCursorY() = 0;
	virtual void    SetTitle( const char *title ) = 0;
	virtual void    SetSize( int width, int height ) = 0;

protected:
	//-------------------------------------------------------------------
	// protected variables
	//-------------------------------------------------------------------
	/* NOTHING */

	//-------------------------------------------------------------------
	// protected methods
	//-------------------------------------------------------------------
	DemoApp() {}
	virtual ~DemoApp() {}
	DemoApp( const DemoApp &app ) {}
	DemoApp& operator = ( const DemoApp &app ) { return (*this); }

private:
	//-------------------------------------------------------------------
	// private variables
	//-------------------------------------------------------------------
	/* NOTHING */

	//-------------------------------------------------------------------
	// private methods
	//-------------------------------------------------------------------
	/* NOTHING */
};

} // namespace SampleProgram
} // namespace Asura
} // namespace Schaft

#endif // __ASURA_DEMO_APP_H__