//-----------------------------------------------------------------------
// File : MRTRenderer.h
// Desc : Multiple Rendering Target Sample Program
// Copyright(c) Project Asura All right reserved.
//-----------------------------------------------------------------------

//-----------------------------------------------------------------------
// History
//-----------------------------------------------------------------------
// <---> 2010/12/05 [Pocol] VK쐬.
// <End>

//-----------------------------------------------------------------------
// Includes
//-----------------------------------------------------------------------
#include <GL/glew.h>
#include <GL/glut.h>
#include "MRTRenderer.h"


//-----------------------------------------------------------------------
// Using Statements
//-----------------------------------------------------------------------
using namespace Schaft::Asura::Graphics;


//-----------------------------------------------------------------------
// Linker
//-----------------------------------------------------------------------
#pragma comment( lib, "glew32.lib" )


namespace Schaft {
namespace Asura {
namespace SampleProgram {

/////////////////////////////////////////////////////////////////////////
// MRTRenderer Implementation
/////////////////////////////////////////////////////////////////////////

///----------------------------------------------------------------------
///<summary>
///RXgN^
///</summary>
///----------------------------------------------------------------------
MRTRenderer::MRTRenderer()
{
	mFrameBuffer = 0;
    mRenderBuffer = 0;
    for( int i=0; i<4; i++ )
    {
        mTexture[i] = 0;
    }
}

///----------------------------------------------------------------------
///<summary>
///fXgN^
///</summary>
///----------------------------------------------------------------------
MRTRenderer::~MRTRenderer()
{
	/* DO NOTHING */
}

///----------------------------------------------------------------------
///<summary>
///
///</summary>
///----------------------------------------------------------------------
void MRTRenderer::Release()
{
    if ( mFrameBuffer )
    {
        glDeleteFramebuffersEXT( 1, &mFrameBuffer );
        mFrameBuffer = 0;
    }
    if ( mRenderBuffer )
    {
        glDeleteRenderbuffersEXT( 1, &mRenderBuffer );
        mRenderBuffer = 0;
    }
	glDeleteTextures( 4, mTexture );
}

///----------------------------------------------------------------------
///<summary>
///
///</summary>
///----------------------------------------------------------------------
bool MRTRenderer::Initialize()
{
    const int WIDTH = 512;
    const int HEIGHT = 512;

    //@GLEW̏
    if ( glewInit() != GLEW_OK )
    {
        return false;
    }

    //@eNX`4
	glGenTextures( 4, mTexture );

    //@4̐ݒĂ
    glEnable( GL_TEXTURE_2D );
    for( int i=0; i<4; i++ )
    {
        glBindTexture( GL_TEXTURE_2D, mTexture[i] );
        glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
    	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
    	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE );
    	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE );
        glTexImage2D( GL_TEXTURE_2D, 0, GL_RGBA, WIDTH, HEIGHT, 0, GL_RGBA, GL_UNSIGNED_BYTE, 0 );
        glBindTexture( GL_TEXTURE_2D, 0 );
    }
    glDisable( GL_TEXTURE_2D );

    //@_[obt@̐ݒ
    glGenRenderbuffersEXT( 1, &mRenderBuffer );
    glBindRenderbufferEXT( GL_RENDERBUFFER_EXT, mRenderBuffer );
    glRenderbufferStorageEXT( GL_RENDERBUFFER_EXT, GL_DEPTH_COMPONENT24, WIDTH, HEIGHT );

    //@t[obt@̐ݒ
    glGenFramebuffersEXT( 1, &mFrameBuffer );
    glBindFramebufferEXT( GL_FRAMEBUFFER_EXT, mFrameBuffer );
    glFramebufferTexture2DEXT( GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, GL_TEXTURE_2D, mTexture[0], 0 );
    glFramebufferTexture2DEXT( GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT1_EXT, GL_TEXTURE_2D, mTexture[1], 0 );
    glFramebufferTexture2DEXT( GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT2_EXT, GL_TEXTURE_2D, mTexture[2], 0 );
    glFramebufferTexture2DEXT( GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT3_EXT, GL_TEXTURE_2D, mTexture[3], 0 );
    glFramebufferRenderbufferEXT( GL_FRAMEBUFFER_EXT, GL_DEPTH_ATTACHMENT_EXT, GL_RENDERBUFFER_EXT, mRenderBuffer );
    glBindFramebufferEXT( GL_FRAMEBUFFER_EXT, 0 );

    //@VF[_C^[tF[X
    mpEffect = CreateCgEffect();
    if ( mpEffect == null )
    {
        return false;
    }

    //@VF[_ǂݍ
    if ( !mpEffect->CreateFromFile( "Contents/Effects/MRTSample.cgfx", null ) )
    {
        return false;
    }

	return true;
}

///----------------------------------------------------------------------
///<summary>
///_OȌ
///</summary>
///----------------------------------------------------------------------
void MRTRenderer::PreRender()
{
	/* DO NOTHING */
}

///----------------------------------------------------------------------
///<summary>
///_O
///</summary>
///----------------------------------------------------------------------
void MRTRenderer::Render()
{
    //@t[obt@oCh
    glBindFramebufferEXT( GL_FRAMEBUFFER_EXT, mFrameBuffer );

    //@4ɏo
    GLuint attachments[4] = 
    {
        GL_COLOR_ATTACHMENT0_EXT,
        GL_COLOR_ATTACHMENT1_EXT,
        GL_COLOR_ATTACHMENT2_EXT,
        GL_COLOR_ATTACHMENT3_EXT,
    }; 
    glDrawBuffers( 4, attachments );

    //@r[|[gݒ
    glViewport( 0, 0, 512, 512 );

    //@FBÕobt@NA
    glClearColor( 100.0/255.0f, 149.0f/255.0f, 237.0f/255.0f, 1.0f );
    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
 
    //@VF[_ŕ`
    for( unsigned int i=0; i<mpEffect->GetNumPasses( 0 ); ++i )
    {
        cgSetPassState( mpEffect->GetPass( 0, i ) );
    	glBegin( GL_TRIANGLES );
    	glVertex3f( -0.75f, -0.75f, 0.0f );
    	glVertex3f( 0.0f, 0.75f, 0.0f );
    	glVertex3f( 0.75f, -0.75f, 0.0f );
    	glEnd();
        cgResetPassState( mpEffect->GetPass( 0, i ) );
    }

    //@r[|[gɖ߂
    int width = SampleProgram::DemoApp::GetInstance()->GetWidth();
    int height =  SampleProgram::DemoApp::GetInstance()->GetHeight();
    glViewport( 0, 0, width, height );

    //@t[obt@̃AoCh
    glBindFramebufferEXT( GL_FRAMEBUFFER_EXT, 0 );
}

///----------------------------------------------------------------------
///<summary>
///ߏKvƂ郌_O
///</summary>
///----------------------------------------------------------------------
void MRTRenderer::RenderAlpha()
{
	/* DO NOTHING */
}

///----------------------------------------------------------------------
///<summary>
///_Ȍ
///</summary>
///----------------------------------------------------------------------
void MRTRenderer::PostRender()
{
    //@eNX`L
    glEnable( GL_TEXTURE_2D );

    // COLOR0
    glColor4f( 1.0f, 1.0f, 1.0f, 1.0f );
    glBindTexture( GL_TEXTURE_2D, mTexture[0] );
    glBegin( GL_QUADS );
    glTexCoord2f( 1.0f, 1.0f ); glVertex3f( -1.0f, 1.0f, 0.0f );
    glTexCoord2f( 1.0f, 0.0f ); glVertex3f( -1.0f, 0.0f, 0.0f );
    glTexCoord2f( 0.0f, 0.0f ); glVertex3f( 0.0f, 0.0f, 0.0f );
    glTexCoord2f( 0.0f, 1.0f ); glVertex3f( 0.0f, 1.0f, 0.0f );
    glEnd();
    glBindTexture( GL_TEXTURE_2D, 0 );

    // COLOR1
    glBindTexture( GL_TEXTURE_2D, mTexture[1] );
    glBegin( GL_QUADS );
    glTexCoord2f( 1.0f, 1.0f ); glVertex3f( 0.0f, 1.0f, 0.0f );
    glTexCoord2f( 1.0f, 0.0f ); glVertex3f( 0.0f, 0.0f, 0.0f );
    glTexCoord2f( 0.0f, 0.0f ); glVertex3f( 1.0f, 0.0f, 0.0f );
    glTexCoord2f( 0.0f, 1.0f ); glVertex3f( 1.0f, 1.0f, 0.0f );
    glEnd();
    glBindTexture( GL_TEXTURE_2D, 0 );

    // COLOR2
    glBindTexture( GL_TEXTURE_2D, mTexture[2] );
    glBegin( GL_QUADS );
    glTexCoord2f( 1.0f, 1.0f );  glVertex3f( -1.0f, 0.0f, 0.0f );
    glTexCoord2f( 1.0f, 0.0f ); glVertex3f( -1.0f, -1.0f, 0.0f );
    glTexCoord2f( 0.0f, 0.0f ); glVertex3f( 0.0f, -1.0f, 0.0f );
    glTexCoord2f( 0.0f, 1.0f );  glVertex3f( 0.0f, 0.0f, 0.0f );
    glEnd();
    glBindTexture( GL_TEXTURE_2D, 0 );
    
    // COLOR3
    glBindTexture( GL_TEXTURE_2D, mTexture[3] );
    glBegin( GL_QUADS );
    glTexCoord2f( 1.0f, 1.0f );  glVertex3f( 0.0f, 0.0f, 0.0f );
    glTexCoord2f( 1.0f, 0.0f ); glVertex3f( 0.0f, -1.0f, 0.0f );
    glTexCoord2f( 0.0f, 0.0f ); glVertex3f( 1.0f, -1.0f, 0.0f );
    glTexCoord2f( 0.0f, 1.0f );  glVertex3f( 1.0f, 0.0f, 0.0f );
    glEnd();
    glBindTexture( GL_TEXTURE_2D, 0 );

    //@eNX`
    glDisable( GL_TEXTURE_2D );

}


} // namespace SampleProgram
} // namespace Asura
} // namespace Schaft

// End Of File.